@@ -14,6 +14,7 @@
 package hugolib
 
 import (
+	"path/filepath"
 	"strings"
 	"testing"
 )
@@ -49,16 +50,16 @@ title: "p3"
 ### p3-h2
 {{% withmarkdown %}}
 {{< level3 >}}
--- layouts/shortcodes/include.html --
+-- layouts/_shortcodes/include.html --
 {{ $p := site.GetPage (.Get 0) }}
 {{ $p.RenderShortcodes }}
--- layouts/shortcodes/withhtml.html --
+-- layouts/_shortcodes/withhtml.html --
 <div>{{ .Page.Title }} withhtml</div>
--- layouts/shortcodes/withmarkdown.html --
+-- layouts/_shortcodes/withmarkdown.html --
 #### {{ .Page.Title }} withmarkdown
--- layouts/shortcodes/level3.html --
+-- layouts/_shortcodes/level3.html --
 Level 3: {{ .Page.Title }}
--- layouts/_default/single.html --
+-- layouts/single.html --
 Fragments: {{ .Fragments.Identifiers }}|
 HasShortcode Level 1: {{ .HasShortcode "include" }}|
 HasShortcode Level 2: {{ .HasShortcode "withmarkdown" }}|
@@ -69,6 +70,7 @@ Content: {{ .Content }}|
 
 	b := Test(t, files)
 
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/p1/index.html",
 		"Fragments: [p1-h1 p2-h1 p2-h2 p2-h3 p2-withmarkdown p3-h1 p3-h2 p3-withmarkdown]|",
 		"HasShortcode Level 1: true|",
@@ -98,16 +100,16 @@ title: "p2"
 ---
 ### p2-h1
 {{% myshort %}}
--- layouts/shortcodes/include.html --
+-- layouts/_shortcodes/include.html --
 {{ $p := site.GetPage (.Get 0) }}
 {{ $p.RenderShortcodes }}
--- layouts/shortcodes/myshort.html --
+-- layouts/_shortcodes/myshort.html --
 Myshort HTML.
--- layouts/shortcodes/myshort.json --
+-- layouts/_shortcodes/myshort.json --
 Myshort JSON.
--- layouts/_default/single.html --
+-- layouts/single.html --
 HTML: {{ .Content }}
--- layouts/_default/single.json --
+-- layouts/single.json --
 JSON: {{ .Content }}
 
 
@@ -115,6 +117,7 @@ JSON: {{ .Content }}
 
 	b := Test(t, files)
 
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/p1/index.html", "Myshort HTML")
 	b.AssertFileContent("public/p1/index.json", "Myshort JSON")
 }
@@ -138,18 +141,20 @@ title: "p2"
 ---
 ### p2-h1
 {{% myshort %}}
--- layouts/shortcodes/include.html --
+-- layouts/_shortcodes/include.html --
 {{ $p := site.GetPage (.Get 0) }}
 {{ $p.RenderShortcodes }}
--- layouts/shortcodes/myshort.html --
+-- layouts/_shortcodes/myshort.html --
 Myshort Original.
--- layouts/_default/single.html --
+-- layouts/single.html --
  {{ .Content }}
 `
 	b := TestRunning(t, files)
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/p1/index.html", "Myshort Original.")
 
-	b.EditFileReplaceAll("layouts/shortcodes/myshort.html", "Original", "Edited").Build()
+	b.EditFileReplaceAll("layouts/_shortcodes/myshort.html", "Original", "Edited").Build()
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/p1/index.html", "Myshort Edited.")
 }
 
@@ -172,12 +177,12 @@ title: "p2"
 ---
 ### Original
 {{% myshort %}}
--- layouts/shortcodes/include.html --
+-- layouts/_shortcodes/include.html --
 {{ $p := site.GetPage (.Get 0) }}
 {{ $p.RenderShortcodes }}
--- layouts/shortcodes/myshort.html --
+-- layouts/_shortcodes/myshort.html --
 Myshort Original.
--- layouts/_default/single.html --
+-- layouts/single.html --
  {{ .Content }}
 
 
@@ -192,12 +197,14 @@ Myshort Original.
 		},
 	).Build()
 
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/p1/index.html", "Original")
 
 	b.EditFileReplaceFunc("content/p2.md", func(s string) string {
 		return strings.Replace(s, "Original", "Edited", 1)
 	})
 	b.Build()
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/p1/index.html", "Edited")
 }
 
@@ -220,12 +227,12 @@ title: "p2"
 ---
 ### Original
 {{% myshort %}}
--- layouts/shortcodes/include.html --
+-- layouts/_shortcodes/include.html --
 {{ $p := .Page.GetPage (.Get 0) }}
 {{ $p.RenderShortcodes }}
--- layouts/shortcodes/myshort.html --
+-- layouts/_shortcodes/myshort.html --
 Myshort Original.
--- layouts/_default/list.html --
+-- layouts/list.html --
  {{ .Content }}
 
 
@@ -233,8 +240,10 @@ Myshort Original.
 `
 	b := TestRunning(t, files)
 
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/mysection/index.html", "p1-h1")
 	b.EditFileReplaceAll("content/mysection/_index.md", "p1-h1", "p1-h1 Edited").Build()
+	b.AssertNoRenderShortcodesArtifacts()
 	b.AssertFileContent("public/mysection/index.html", "p1-h1 Edited")
 }
 
@@ -244,20 +253,20 @@ func TestRenderShortcodesNestedPageContextIssue12356(t *testing.T) {
 	files := `
 -- hugo.toml --
 disableKinds = ["taxonomy", "term", "rss", "sitemap", "robotsTXT", "404"]
--- layouts/_default/_markup/render-image.html --
+-- layouts/_markup/render-image.html --
 {{- with .PageInner.Resources.Get .Destination -}}Image: {{ .RelPermalink }}|{{- end -}}
--- layouts/_default/_markup/render-link.html --
+-- layouts/_markup/render-link.html --
 {{- with .PageInner.GetPage .Destination -}}Link: {{ .RelPermalink }}|{{- end -}}
--- layouts/_default/_markup/render-heading.html --
+-- layouts/_markup/render-heading.html --
 Heading: {{ .PageInner.Title }}: {{ .PlainText }}|
--- layouts/_default/_markup/render-codeblock.html --
+-- layouts/_markup/render-codeblock.html --
 CodeBlock: {{ .PageInner.Title }}: {{ .Type }}|
--- layouts/_default/list.html --
+-- layouts/list.html --
 Content:{{ .Content }}|
 Fragments: {{ with .Fragments }}{{.Identifiers }}{{ end }}|
--- layouts/_default/single.html --
+-- layouts/single.html --
 Content:{{ .Content }}|
--- layouts/shortcodes/include.html --
+-- layouts/_shortcodes/include.html --
 {{ with site.GetPage (.Get 0) }}
   {{ .RenderShortcodes }}
 {{ end }}
@@ -314,6 +323,8 @@ iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVR42mNkYPhfDwAChwGA60e6kgAA
 
 	b := Test(t, files)
 
+	b.AssertNoRenderShortcodesArtifacts()
+
 	b.AssertFileContent("public/markdown/index.html",
 		// Images.
 		"Image: /posts/p1/pixel1.png|\nImage: /posts/p1/pixel2.png|\n|\nImage: /markdown/pixel3.png|</p>\n|",
@@ -333,3 +344,186 @@ iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVR42mNkYPhfDwAChwGA60e6kgAA
 
 	b.AssertFileContent("public/html/index.html", "! hugo_ctx")
 }
+
+// Issue 12854.
+func TestRenderShortcodesWithHTML(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableLiveReload = true
+disableKinds = ["home", "taxonomy", "term"]
+markup.goldmark.renderer.unsafe = true
+-- content/p1.md --
+---
+title: "p1"
+---
+{{% include "p2" %}}
+-- content/p2.md --
+---
+title: "p2"
+---
+Hello <b>world</b>. Some **bold** text. Some Unicode: 神真美好.
+-- layouts/_shortcodes/include.html --
+{{ with site.GetPage (.Get 0) }}
+<div>{{ .RenderShortcodes }}</div>
+{{ end }}
+-- layouts/single.html --
+{{ .Content }}
+`
+
+	b := TestRunning(t, files, TestOptWarn())
+
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertLogContains(filepath.ToSlash("WARN  .RenderShortcodes detected inside HTML block in \"/content/p1.md\"; this may not be what you intended, see https://gohugo.io/methods/page/rendershortcodes/#limitations\nYou can suppress this warning by adding the following to your project configuration:\nignoreLogs = ['warning-rendershortcodes-in-html']"))
+	b.AssertFileContent("public/p1/index.html", "<div>Hello <b>world</b>. Some **bold** text. Some Unicode: 神真美好.\n</div>")
+	b.EditFileReplaceAll("content/p2.md", "Hello", "Hello Edited").Build()
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContent("public/p1/index.html", "<div>Hello Edited <b>world</b>. Some **bold** text. Some Unicode: 神真美好.\n</div>")
+}
+
+func TestRenderShortcodesIncludeMarkdownFileWithoutTrailingNewline(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableLiveReload = true
+disableKinds = ["home", "taxonomy", "term"]
+markup.goldmark.renderer.unsafe = true
+-- content/p1.md --
+---
+title: "p1"
+---
+Content p1 id-1000.{{% include "p2" %}}{{% include "p3" %}}
+
+§§§ go
+code_p1
+§§§
+§§§ go
+code_p1_2
+§§§
+
+§§§ go
+code_p1_3
+§§§
+-- content/p2.md --
+---
+title: "p2"
+---
+§§§ bash
+code_p2
+§§§
+Foo.
+-- content/p3.md --
+---
+title: "p3"
+---
+§§§ php
+code_p3
+§§§
+-- layouts/_shortcodes/include.html --
+{{ with site.GetPage (.Get 0) -}}
+{{ .RenderShortcodes -}}
+{{ end -}}
+-- layouts/single.html --
+{{ .Content }}
+-- layouts/_markup/render-codeblock.html --
+<code>{{ .Inner | safeHTML }}</code>
+`
+
+	b := TestRunning(t, files, TestOptWarn())
+
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContentEquals("public/p1/index.html", "<p>Content p1 id-1000.</p>\n<code>code_p2</code><p>Foo.</p>\n<code>code_p3</code><code>code_p1</code><code>code_p1_2</code><code>code_p1_3</code>")
+	b.EditFileReplaceAll("content/p1.md", "id-1000.", "id-100.").Build()
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContentEquals("public/p1/index.html", "<p>Content p1 id-100.</p>\n<code>code_p2</code><p>Foo.</p>\n<code>code_p3</code><code>code_p1</code><code>code_p1_2</code><code>code_p1_3</code>")
+	b.EditFileReplaceAll("content/p2.md", "code_p2", "codep2").Build()
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContentEquals("public/p1/index.html", "<p>Content p1 id-100.</p>\n<code>codep2</code><p>Foo.</p>\n<code>code_p3</code><code>code_p1</code><code>code_p1_2</code><code>code_p1_3</code>")
+	b.EditFileReplaceAll("content/p3.md", "code_p3", "code_p3_edited").Build()
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContentEquals("public/p1/index.html", "<p>Content p1 id-100.</p>\n<code>codep2</code><p>Foo.</p>\n<code>code_p3_edited</code><code>code_p1</code><code>code_p1_2</code><code>code_p1_3</code>")
+}
+
+// Issue 13004.
+func TestRenderShortcodesIncludeShortRefEdit(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableLiveReload = true
+disableKinds = ["home", "taxonomy", "term", "section", "rss", "sitemap", "robotsTXT", "404"]
+-- content/first/p1.md --
+---
+title: "p1"
+---
+## p1-h1
+{{% include "p2" %}}
+-- content/second/p2.md --
+---
+title: "p2"
+---
+### p2-h1
+
+This is some **markup**.
+-- layouts/_shortcodes/include.html --
+{{ $p := site.GetPage (.Get 0) -}}
+{{ $p.RenderShortcodes -}}
+-- layouts/single.html --
+{{ .Content }}
+`
+	b := TestRunning(t, files)
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContentEquals("public/first/p1/index.html", "<h2 id=\"p1-h1\">p1-h1</h2>\n<h3 id=\"p2-h1\">p2-h1</h3>\n<p>This is some <strong>markup</strong>.</p>\n")
+	b.EditFileReplaceAll("content/second/p2.md", "p2-h1", "p2-h1-edited").Build()
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContentEquals("public/first/p1/index.html", "<h2 id=\"p1-h1\">p1-h1</h2>\n<h3 id=\"p2-h1-edited\">p2-h1-edited</h3>\n<p>This is some <strong>markup</strong>.</p>\n")
+}
+
+// Issue 13051.
+func TestRenderShortcodesEmptyParagraph(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableKinds = ['section','rss','sitemap','taxonomy','term']
+-- layouts/home.html --
+{{ .Content }}
+-- layouts/single.html --
+{{ .Content }}
+-- layouts/_shortcodes/include.html --
+ {{ with site.GetPage (.Get 0) }}
+  {{ .RenderShortcodes }}
+{{ end }}
+-- content/_index.md --
+---
+title: home
+---
+
+a
+
+{{% include "/snippet" %}}
+
+b
+
+-- content/snippet.md --
+---
+title: snippet
+build:
+  render: never
+  list: never
+---
+
+_emphasized_
+
+not emphasized
+
+`
+
+	b := Test(t, files)
+	b.AssertNoRenderShortcodesArtifacts()
+	b.AssertFileContentEquals("public/index.html",
+		"<p>a</p>\n<p><em>emphasized</em></p>\n<p>not emphasized</p>\n<p>b</p>\n",
+	)
+}
